import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Boss battle of Kirby Run. The boss moves and fires attacks at Kirby at
 * random time intervals.
 * 
 * @Jasmine Xiong 
 * @June 11, 2019
 */
public class Boss extends Actor
{
    // instance variables
    HealthBar hpBar;
    int hp;
    int maxhp;

    // jumping variables
    int vspeed = 0;
    int gravity = 1;
    int jumpTimer = 0;
    
    // shooting variables
    int cooldown = 50;
    
    public Boss(){
        // sets health bar details
        maxhp = 500;
        hp = maxhp;
        hpBar = new HealthBar (hp, this);
    }
    
    public void addedToWorld(World w){
        // adds the health bar to world
        w.addObject(hpBar, 300, 100);
        hpBar.update(maxhp);
    }
    
    int animateTimer = 0;
    public void act() 
    {
        // use of the health bar class
        checkHealth();
        
        // self explanatory
        jumpRandomly();
        checkFall();
        
        // spawns bullets but cooldown prevents bullets being to close to each other
        cooldown--;
        spawnBullets();
        
        // animations!
        animateTimer++;
        animate();
    }    
    
    public void checkHealth(){
        // if hp is above zero and intersects with fireball, reduce health
        if(hp > 0){
            Fireball b = (Fireball)getOneIntersectingObject(Fireball.class);
            if(b != null){
                getWorld().removeObject(b);
                hp -= (int)(maxhp * 0.10);
                hpBar.update(hp);
            }
        }
        // otherwise health would be zero so the game would be over
        else{
            Greenfoot.setWorld(new EndScreen());
        }
    }
    
    // animations!
    public void animate(){
        if(animateTimer == 5) setImage ("bosss0.png");
        if(animateTimer == 10) setImage ("bosss1.png");
        if(animateTimer == 15) setImage ("bosss2.png");
        if(animateTimer == 20) setImage ("bosss3.png");
        if(animateTimer == 25){
            setImage ("bosss2.png");
            animateTimer = 0;
        }
    }
    
    // checks if Boss is on ground
    public boolean onGround(){
        Ground g = (Ground)getOneIntersectingObject(Ground.class);
        return g != null;
    }
    
    // fall to ground if Boss isn't touching the ground
    public void checkFall(){
        if(onGround()){
            vspeed = 0;
        }
        else{
            fall();
        }
    }
    
    // falls to the ground if Boss is not touching the ground
    public void fall(){
        setLocation(getX(), getY() + vspeed);
        vspeed = vspeed + gravity;
    }
    
    // basic jumping but at random intervals
    public void jumpRandomly(){
        if(Greenfoot.getRandomNumber(70) == 1 && onGround()){
            vspeed = -18;
            setLocation(getX(), getY() + vspeed);
            vspeed = vspeed + gravity;
        }
    }
    
    // bullets shot from Boss shoot at Kirby randomly
    public void spawnBullets(){
        if(Greenfoot.getRandomNumber(30) == 1 && cooldown < 0){
            getWorld().addObject(new Bullets(), 450, getY());
            cooldown = 50;
        }
    }
}
