import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * THIS IS THE MAIN WORLD SO DOCUMENTATION IS HERE
 * 
 * CREDITS
 * Sound: Songily
 * Images: The Spriters Resource, Google Images, Lunapic (used for editing)
 * 
 * INSTRUCTIONS
 * Controls
 * Space bar - makes Kirby jump
 * Enter key - makes Kirby shoot a fireball
 * 
 * Bullets - fire towards Kirby at random times
 * Enemies - run towards Kirby at random times and speeds
 * Boss - will shoot randomly shoot bullets and randomly jump, can only
 *        be defeated when his health bar runs out
 *        
 * Score - just keeps track of time that Kirby has ran for, once he reaches
 *         a certain time, you can move on to the next level
 *        
 * KNOWN BUGS/ISSUES
 * Music doesn't stop when you press the pause button. I'm not sure how to
 * fix this :(
 * Kirby's shooting motion is a little sketchy. The shooting pose only
 * sets when you release the enter key. It doesn't affect the progression
 * of the game but sometimes it's just not visually appealing.
 * Also not sure if this is an issue but music only works when you start
 * at the TitleScreen.
 * 
 * Main gameplay world. (Level 1)
 * Kirby must jump over bullets and shoot at enemies.
 * Kirby advances to the next level when the score hits 500.
 * 
 * @Jasmine Xiong
 * @May 29, 2019
 */
public class MyWorld extends World
{
    // fireballTimer makes sure Kirby doesn't constantly shoot
    int fireballTimer = 0;
    // cooldown makes sure bullets aren't spawn too close to each other
    int cooldown = 50;
    public MyWorld()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(600, 400, 1); 
        GreenfootImage image = new GreenfootImage("Webp.net-compress-image.jpg");
        image.scale(image.getWidth()-800, image.getHeight()-320);
        setBackground(image);
        
        // add ground and Kirby to world
        Ground g = new Ground();
        addObject(g, 300, 370);
        Kirby k = new Kirby();
        addObject(k, 100, 310);
        
        // scoreboard that increases every act
        Score score = new Score();
        addObject(score, 70, 50);
    }
    
    public void act(){
        spawnBullets();
        spawnEnemies();
        fireballTimer++;
    }
    
    public void spawnEnemies(){
        // enemies at random
        if(Greenfoot.getRandomNumber(150) == 1){
            addObject(new Enemy(), 600, 297);
        }
    }
    
    public void spawnBullets(){
        // bullets at random
        cooldown--;
        if(Greenfoot.getRandomNumber(50)==1 && cooldown < 0){
            addObject(new Bullets(), 600, 300);
            cooldown = 50;
        }
    }
}
